@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from abc import ABC
-from typing import Any, Dict, Generic, Iterable, Sequence, Set, TypeVar, Union
+from typing import Any, Dict, Generic, Iterable, Iterator, Mapping, Sequence, Set, TypeVar, Union
 from uuid import UUID
 
 import instancelib as ins
@@ -33,7 +33,25 @@ class AbstractMemoryEnvironment(AbstractEnvironment[IT, KT, DT, VT, RT, LT],
     _truth: LabelProvider[KT, LT]
     _logger: BaseLogger[KT, LT, Any]
     _named_providers: Dict[str, InstanceProvider[IT, KT, DT, VT, RT]]
-   
+
+    def __contains__(self, __o: object) -> bool:
+        return __o in self._named_providers
+
+    def __getitem__(self, __k: str) -> InstanceProvider[IT, KT, DT, VT, RT]:
+        return self._named_providers[__k]
+
+    def __setitem__(self, __k: str, __v: InstanceProvider[IT, KT, DT, VT, RT]) -> None:
+        self.set_named_provider(__k, __v)
+
+    def __len__(self) -> int:
+        return len(self._named_providers)
+
+    def __delitem__(self, __v: str) -> None:
+        del self._named_providers[__v]
+
+    def __iter__(self) -> Iterator[str]:
+        return iter(self._named_providers)
+
     @property
     def dataset(self) -> InstanceProvider[IT, KT, DT, VT, RT]:
         return self._public_dataset
@@ -71,8 +89,8 @@ class AbstractMemoryEnvironment(AbstractEnvironment[IT, KT, DT, VT, RT, LT],
     def set_named_provider(self, name: str, value: InstanceProvider[IT, KT, DT, VT, RT]):
         self._named_providers[name] = value
 
-    def create_named_provider(self, name: str) -> InstanceProvider[IT, KT, DT, VT, RT]:
-        self._named_providers[name] = self.create_empty_provider()
+    def create_named_provider(self, name: str, keys: Iterable[KT] = list()) -> InstanceProvider[IT, KT, DT, VT, RT]:
+        self._named_providers[name] = self.create_bucket(keys)
         return self._named_providers[name]   
     
 class MemoryEnvironment(
@@ -148,7 +166,7 @@ class MemoryEnvironment(
     def from_instancelib_simulation(cls, 
                          environment: ins.AbstractEnvironment[IT, KT, DT, VT, RT, LT]
                          )-> AbstractEnvironment[IT, KT, DT, VT, RT, LT]:
-        dataset = environment.all_datapoints
+        dataset = environment.all_instances
         unlabeled = MemoryBucketProvider(dataset, dataset.key_list)
         labeled = MemoryBucketProvider(dataset, [])
         labels = MemoryLabelProvider[KT, LT].from_data(
